/*
 * $Id: libbridge_if.c,v 1.3 2000/01/19 12:37:46 root Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include "libbridge.h"

struct bridge *br_find_bridge(char *brname)
{
	struct bridge *b;

	b = bridge_list;
	while (b != NULL) {
		if (!strcmp(b->ifname, brname))
			return b;

		b = b->next;
	}

	return NULL;
}

void br_add_bridge(char *brname)
{
	unsigned long arg[2];
	char _br[IFNAMSIZ];

	memcpy(_br, brname, IFNAMSIZ);
	arg[0] = BRCTL_ADD_BRIDGE;
	arg[1] = (unsigned long)_br;
	if (ioctl(br_socket_fd, SIOCGIFBR, arg) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case EEXIST:
		fprintf(stderr,	"device %s already exists; can't create "
			"bridge with the same name\n", brname);
		break;

	default:
		perror("br_add_bridge");
		exit(-1);
	}
}

void br_del_bridge(char *brname)
{
	unsigned long arg[2];
	char _br[IFNAMSIZ];

	memcpy(_br, brname, IFNAMSIZ);
	arg[0] = BRCTL_DEL_BRIDGE;
	arg[1] = (unsigned long)_br;
	if (ioctl(br_socket_fd, SIOCGIFBR, arg) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case ENXIO:
		fprintf(stderr, "bridge %s doesn't exist; can't delete it\n", brname);
		break;

	case EBUSY:
		fprintf(stderr, "bridge %s is still up; can't delete it\n", brname);
		break;

	default:
		perror("br_del_bridge");
		exit(-1);
	}
}

void br_add_interface(struct bridge *br, int ifindex)
{
	char ifname[IFNAMSIZ];
	struct ifreq ifr;
	unsigned long *i;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_ADD_IF;
	i[1] = ifindex;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case EBUSY:
		fprintf(stderr,	"device %s is already a member of a bridge; "
			"can't enslave it to bridge %s.\n",
			if_indextoname(ifindex, ifname), br->ifname);
		break;

	default:
		perror("br_add_interface");
		exit(-1);
	}
}

void br_del_interface(struct bridge *br, int ifindex)
{
	char ifname[IFNAMSIZ];
	struct ifreq ifr;
	unsigned long *i;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_DEL_IF;
	i[1] = ifindex;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case EINVAL:
		fprintf(stderr, "device %s is not a slave of %s\n",
			if_indextoname(ifindex, ifname), br->ifname);
		break;

	default:
		perror("br_del_interface");
		exit(-1);
	}
}

void br_set_bridge_forward_delay(struct bridge *br, int forward_delay)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_BRIDGE_FORWARD_DELAY;
	i[1] = forward_delay;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_bridge_forward_delay");
		return;
	}
}

void br_set_bridge_hello_time(struct bridge *br, int hello_time)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_BRIDGE_HELLO_TIME;
	i[1] = hello_time;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_bridge_hello_time");
		return;
	}
}

void br_set_bridge_max_age(struct bridge *br, int max_age)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_BRIDGE_MAX_AGE;
	i[1] = max_age;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_bridge_max_age");
		return;
	}
}

void br_set_ageing_time(struct bridge *br, int ageing_time)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_AGEING_TIME;
	i[1] = ageing_time;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_bridge_max_age");
		return;
	}
}

void br_set_gc_interval(struct bridge *br, int gc_interval)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_GC_INTERVAL;
	i[1] = gc_interval;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_bridge_max_age");
		return;
	}
}

void br_set_stp_state(struct bridge *br, int stp_state)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_BRIDGE_STP_STATE;
	i[1] = stp_state;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_stp_state");
		return;
	}
}

void br_set_bridge_priority(struct bridge *br, int bridge_priority)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_BRIDGE_PRIORITY;
	i[1] = bridge_priority;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_stp_state");
		return;
	}
}

void br_set_port_priority(struct port *p, int port_priority)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, p->parent->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_PORT_PRIORITY;
	i[1] = p->index;
	i[2] = port_priority;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_stp_state");
		return;
	}
}

void br_set_path_cost(struct port *p, int path_cost)
{
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, p->parent->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_SET_PATH_COST;
	i[1] = p->index;
	i[2] = path_cost;
	if (ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr) < 0) {
		perror("br_set_stp_state");
		return;
	}
}

int br_get_fdb_list(struct bridge *br, struct fdb_entry *fdbs, int num)
{
	int err;
	unsigned long *i;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	i = (unsigned long *)(&ifr.ifr_data);
	i[0] = BRCTL_GET_FDB_ENTRIES;
	i[1] = (unsigned long)fdbs;
	i[2] = num;
	if ((err = ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr)) < 0) {
		perror("br_get_fdb_list");
		return 0;
	}

	return err;
}
